# $Id: Greenwich.pm,v 1.13 2003/12/05 13:45:52 jodrell Exp $
package Greenwich;

use Gnome2;
use Greenwich::L10N;
use Greenwich::WhoisMap;
use Greenwich::WhoisClient;
use Greenwich::AboutBox;
use Greenwich::ServerList;
use Greenwich::Preferences;
use Gtk2::Helper;
use Data::Dumper;
use vars qw($PREFIX $ICONFILE);
use strict;

our $PREFIX = '/usr/local';

sub new {
	my $self		= {};
	$self->{PACKAGE}	= shift;
	$self->{NAME}		= 'Greenwich';
	$self->{VERSION}	= '0.6.2';
	$self->{RCFILE}		= sprintf('%s/.%src', $ENV{HOME}, $self->{NAME});
	$main::INSTANCES++;
	bless($self, $self->{PACKAGE});

	# should $PREFIX have been modified at runtime, this will
	# reflect that:
	our $ICONFILE = sprintf('%s/share/pixmaps/greenwich.png', $PREFIX);
	return $self;
}

sub init {
	my $self = shift;
	Gnome2::Program->init($self->{NAME}, $self->{VERSION}, 'libgnomeui');
	$self->{client} = Greenwich::WhoisClient->new();
	$self->load_options();
	$self->load_lexicon();
	$self->build_ui();
	$self->configure();
	Gtk2->main();
	return;
}

sub load_lexicon {
	my $self = shift;
	my $lexicon;
	($lexicon = Greenwich::L10N->get_handle()) || ($lexicon = Greenwich::L10N->get_handle('en')) || die "Can't find an acceptable language module";
	$self->{lexicon} = $lexicon;
	return;
}

sub load_options {
	my $self = shift;
	$self->{OPTIONS} = {
		auto	=> 'true',
		follow	=> 'true',
		timeout	=> 10,
	};
	open(RCFILE, $self->{RCFILE}) or return undef;
	while (<RCFILE>) {
		chomp;
		unless (/^\s?([;#])/) {
			my ($name, $value) = split(/=/, $_, 2);
			$self->{OPTIONS}{lc($name)} = $value;
		}
	}
	close(RCFILE);
	return;
}

sub save_options {
	my $self = shift;
	$self->{OPTIONS}{history} = join(',', grep { defined } @{$self->{HISTORY}});
	$self->{OPTIONS}{servers} = join(',', @{$self->{SERVERS}});
	open(RCFILE, ">$self->{RCFILE}") or $self->error("$self->{RCFILE}: $!") and return undef;
	foreach my $name (keys %{$self->{OPTIONS}}) {
		printf(RCFILE "%s=%s\n", lc($name), $self->{OPTIONS}->{$name});
	}
	close(RCFILE);
	return;
}

sub configure {
	my $self = shift;
	@{$self->{SERVERS}} = split(/,/, $self->{OPTIONS}{servers});
	@{$self->{DEFAULT_SERVERS}} = qw(
		whois.networksolutions.com
		whois.ripe.net
		whois.centralnic.net
		whois.nic.us
		whois.nic.uk
		whois.aunic.net
		whois.cnnic.net.cn
		whois.crsnic.net
		whois.krnic.net
		whois.nic.ad.jp
		whois.nic.br
		whois.nic.fr
		whois.nic.nu
		whois.ripe.net
		whois.tonic.to
		whois.twnic.net
	);
	@{$self->{HISTORY}} = grep { defined } split(/,/, $self->{OPTIONS}{history});
	$self->set_output($self->get_output());
	$self->auto_toggle;
	$self->set_servers();
	$self->set_history();

	return;
}

sub build_ui {
	my $self = shift;
	$self->{app} = Gnome2::App->new($self->{NAME}, $self->{NAME});
	$self->{icon} = Gtk2::Gdk::Pixbuf->new_from_file($ICONFILE);
	$self->{app}->set_icon($self->{icon});
	
	$self->{app}->signal_connect('delete_event', sub { $self->close_program() });

	$self->{vbox} = Gtk2::VBox->new(0, 0);

	$self->{tooltip} = Gtk2::Tooltips->new;

	$self->{accel} = Gtk2::AccelGroup->new;
	$self->{app}->add_accel_group($self->{accel});

	$self->build_menu();
	$self->build_toolbar();
	$self->build_controlbar();

	$self->{output} = Gtk2::TextView->new();
	$self->{output}->set_editable(0);
	# this will use the gconf value when the gconf libs actually work:
	$self->{output}->get_buffer->create_tag('monospace', family => 'monospace');
	$self->{window} = Gtk2::ScrolledWindow->new();
	$self->{window}->set_policy('automatic', 'automatic');
	$self->{window}->add_with_viewport($self->{output});

	# set up the user interface settings:
	$self->{controls}{auto}->signal_emit('clicked') if ($self->{OPTIONS}{auto} eq 'true');

	$self->{status} = Gtk2::Statusbar->new;
	$self->{context} = $self->{status}->get_context_id('Main');
	$self->status($self->string('ready'));

	# bring everything together:
	$self->{vbox}->pack_start($self->{window},     1, 1, 0);
	$self->{vbox}->pack_start($self->{status},     0, 1, 0);

	$self->{app}->set_contents($self->{vbox});

	$self->{app}->set_default_size(400, 500);

	$self->{app}->show_all();

	$self->{controls}{query}->entry()->grab_focus();

	return;
}

sub build_menu {
	my $self = shift;
	$self->{app}->create_menus([
		{
			type	=> 'subtree',
			label	=> '_File',
			subtree	=> [
				{
					type		=> 'item',
					label		=> '_New Window',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-new',
					callback	=> sub { Greenwich->new()->init() },
					accelerator_key	=> ord('N'),
					ac_mods		=> ['control-mask'],
				},
				{
					type		=> 'item',
					label		=> '_Save...',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-save',
					callback	=> sub { $self->save_output() },
					accelerator_key	=> ord('S'),
					ac_mods		=> ['control-mask'],
				},
				{	type		=> 'separator' },
				{
					type		=> 'item',
					label		=> '_Close',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-close',
					callback	=> sub { $self->close_program() },
					accelerator_key	=> ord('W'),
					ac_mods		=> ['control-mask'],
				},
				{
					type		=> 'item',
					label		=> 'E_xit',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-quit',
					callback	=> sub { $main::INSTANCES = 1 ; $self->close_program() },
					accelerator_key	=> ord('Q'),
					ac_mods		=> ['control-mask'],
				},
			]
		},
		{
			type	=> 'subtree',
			label	=> '_Edit',
			subtree	=> [
				{
					type		=> 'item',
					label		=> '_Preferences...',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-preferences',
					callback	=> sub { $self->edit_preferences() },
					accelerator_key	=> ord('E'),
					ac_mods		=> ['control-mask'],
				},
				{
					type		=> 'item',
					label		=> '_'.$self->string('serverlist').'...',
					pixmap_type	=> 'stock',
					pixmap_info	=> 'gtk-index',
					callback	=> sub { $self->edit_server_list() },
					accelerator_key	=> ord('L'),
					ac_mods		=> ['control-mask'],
				},
			]
		},
		{
			type	=> 'subtree',
			label	=> '_Help',
			subtree	=> [
				{
					type		=> 'item',
					label		=> '_About...',
					pixmap_type 	=> 'stock',
					pixmap_info	=> 'gtk-about',
					callback	=> sub { Greenwich::AboutBox->new($self) },
					accelerator_key	=> ord('B'),
					ac_mods		=> ['control-mask'],
				},
			]
		}
	]);
	return;
}

sub build_toolbar {
	my $self = shift;
	$self->{toolbar} = $self->{app}->create_toolbar([
		{
			type		=> 'item',
			label		=> 'Save',
			pixmap_type	=> 'stock',
			pixmap_info	=> 'gtk-save',
			hint		=> $self->string('savehint'),
			callback	=> sub { $self->save_output() },
		},
		{
			type		=> 'item',
			label		=> 'Preferences',
			pixmap_type	=> 'stock',
			pixmap_info	=> 'gtk-preferences',
			hint		=> $self->string('prefshint'),
			callback	=> sub { $self->edit_preferences() },
		},
		{
			type		=> 'item',
			label		=> 'Server List',
			pixmap_type	=> 'stock',
			pixmap_info	=> 'gtk-index',
			hint		=> $self->string('serverlist'),
			callback	=> sub { $self->edit_server_list() },
		},
		{
			type		=> 'item',
			label		=> 'Query',
			pixmap_type	=> 'stock',
			pixmap_info	=> 'gtk-jump-to',
			hint		=> $self->string('queryhint'),
			callback	=> sub { $self->make_query() },
		},
	]);
	return;
}


sub build_controlbar {
	my $self = shift;
	$self->{controlbox} = Gtk2::HBox->new(0, 0);
	$self->{controlbox}->set_spacing(5);

	$self->{controls}{auto} = Gtk2::CheckButton->new($self->string('autodetect'));
	$self->{controls}{auto}->signal_connect('clicked', sub { $self->auto_toggle() });
	$self->{controlbox}->pack_start($self->{controls}{auto}, 0, 0, 0);

	$self->{controlbox}->pack_start(Gtk2::VSeparator->new(), 0, 0, 0);

	$self->{server_label} = $self->label('server');
	$self->{controlbox}->pack_start($self->{server_label}, 0, 0, 0);

	$self->{controls}{server} = Gtk2::Combo->new();
	$self->{controls}{server}->disable_activate();
	$self->{controlbox}->pack_start($self->{controls}{server}, 1, 1, 0);

	$self->{controlbox}->pack_start($self->label('port'), 0, 0, 0);

	$self->{port_adj} = Gtk2::Adjustment->new(43, 0, 65535, 1, 10, 1);
	$self->{controls}{port} = Gtk2::SpinButton->new($self->{port_adj}, 1, 0);
	$self->{controlbox}->pack_start($self->{controls}{port}, 0, 0, 0);

	$self->{controlbox}->pack_start($self->label('query'), 0, 0, 0);

	$self->{controls}{query} = Gtk2::Combo->new();
	$self->{controls}{query}->disable_activate();
	$self->{controls}{query}->entry()->signal_connect('activate', sub { $self->make_query() });
	$self->{controlbox}->pack_start($self->{controls}{query}, 1, 1, 0);

	$self->{controls}{button} = Gtk2::Button->new();
	$self->{controls}{button}->add(Gtk2::Image->new_from_stock('gtk-jump-to', 'small-toolbar'));
	$self->{controls}{button}->set_relief('none');
	$self->{controls}{button}->signal_connect('clicked', sub { $self->make_query() });
	$self->{controlbox}->pack_start($self->{controls}{button}, 0, 0, 0);

	$self->{vbox}->pack_start($self->{controlbox}, 0, 1, 0);

	return;
}

sub auto_toggle {
	my $self = shift;
	if ($self->{controls}{auto}->get_active()) {
		$self->{OPTIONS}{auto} = 'true';
		$self->{controls}{server}->hide;
		$self->{server_label}->hide;
	} else {
		$self->{OPTIONS}{auto} = 'false';
		$self->{controls}{server}->show;
		$self->{server_label}->show;
	}
	return;
}

sub server_list {
	my $self = shift;
	my @servers = @{$self->{DEFAULT_SERVERS}};
	if (scalar(@{$self->{SERVERS}}) > 0) {
		push(@servers, @{$self->{SERVERS}});
	}
	return @servers;
}

sub set_servers {
	my $self = shift;
	$self->{controls}{server}->set_popdown_strings($self->server_list());
	$self->{controls}{server}->entry->set_text('');
	return;
}

sub set_history {
	my ($self, $query) = @_;
	my @history = @{$self->{HISTORY}};
	if (defined($query)) {
		unshift(@history, $query);
		$self->{HISTORY} = \@history;
		$self->{controls}{query}->set_popdown_strings(@history);
	} else {
		$self->{controls}{query}->set_popdown_strings(@history);
		$self->{controls}{query}->entry->set_text('');
	}
	return;
}

sub string {
	my ($self, $str) = @_;
	return $self->{lexicon}->maketext($str);
}

sub label {
	my ($self, $str) = @_;
	return Gtk2::Label->new($self->{lexicon}->maketext($str));
}

sub status {
	my ($self, $str) = @_;
	$self->{status}->push($self->{context}, $str);
	return;
}

sub error {
	my ($self, $error) = @_;
	my $dialog = Gtk2::MessageDialog->new($self->{app}, 'modal', 'error', 'ok', $error);
	$dialog->signal_connect('response', sub { $dialog->destroy });
	$dialog->set_icon($self->{icon});
	$dialog->run;
	return;
}

sub make_query {
	my $self = shift;
	my $t0 = time();
	my $query = $self->{controls}{query}->entry()->get_text();
	my %params = (
		query	=> $query,
		port	=> $self->{controls}{port}->get_value_as_int(),
		server	=> ($self->{controls}{auto}->get_active() ? whois_server($query) : $self->{controls}{server}->entry()->get_text()),
		timeout	=> $self->{OPTIONS}{timeout},
		follow	=> $self->{OPTIONS}{follow},
	);
	if ($params{server} eq '') {
		$self->error($self->string('nohost'));
		return undef;
	}
	$self->set_history($query);
	$self->{controls}{query}->entry()->set_text($query);
	$self->status(sprintf($self->string('connecting'), $params{server}));
	Gtk2->main_iteration() while (Gtk2->events_pending());
	my ($response, $error) = $self->{client}->whois(%params);
	if (defined($error)) {
		$self->error($error);
		$self->status($self->string('error'));
	} else {
		$self->status(sprintf($self->string('done'), (time() - $t0)));
	}
	$self->set_output($response);
	return;
}

sub set_output {
	my ($self, $output) = @_;
	$output =~ s/\r//g;
	$self->{output}->get_buffer->set_text('');
	$self->{output}->get_buffer->insert_with_tags_by_name(
		$self->{output}->get_buffer->get_start_iter,
		$output,
		'monospace'
	);

	return;
}

sub get_output {
	my $self = shift;
	return $self->{output}->get_buffer->get_text(
		$self->{output}->get_buffer->get_start_iter,
		$self->{output}->get_buffer->get_end_iter,
		1
	);
}

sub save_output {
	my $self = shift;
	my $output = $self->get_output();
	my $file_selector = Gtk2::FileSelection->new($self->string('selectfile'));
	$file_selector->set_position('center');
	$file_selector->signal_connect('destroy', sub { $file_selector->destroy() });
	$file_selector->cancel_button->signal_connect('clicked', sub { $file_selector->destroy() });
	$file_selector->ok_button->signal_connect('clicked', sub {
		my $file = $file_selector->get_filename();
		$file_selector->destroy();
		if (-e $file) {
			my $question = Gtk2::MessageDialog->new($self->{app}, 'modal', 'question', 'ok-cancel', $self->string('confirmoverwrite'));
			$question->set_icon($self->{icon});
			$question->signal_connect(
				'response',
				sub {
					if ($_[1] == -5) {
						open(FILE, ">$file") or $self->warning("$file: $!") and return undef;
						print FILE $output;
						$self->status(sprintf($self->string('bufferwritten'), $file))
					}
					$question->destroy;
				}
			);
			$question->run;
		} else {
			open(FILE, ">$file") or $self->warning("$file: $!") and return undef;
			print FILE $output;
			$self->status(sprintf($self->string('bufferwritten'), $file))
		}
	});
	$file_selector->show();
	return;
}

sub edit_server_list {
	my $self = shift;
	my $server_list = Greenwich::ServerList->new(lexicon => $self->{lexicon}, app => $self->{app});
	$server_list->set_servers(@{$self->{SERVERS}});
	$server_list->ok_button()->signal_connect('clicked', sub {
		@{$self->{SERVERS}} = $server_list->servers();
		$self->set_servers();
		$server_list->destroy();
	});
	$server_list->show();
	return;
}

sub edit_preferences {
	my $self = shift;
	my $prefs = Greenwich::Preferences->new(lexicon => $self->{lexicon}, options => $self->{OPTIONS});
	$prefs->ok_button()->signal_connect('clicked', sub {
		my %options = $prefs->get_options();
		$self->{OPTIONS} = \%options;
		$self->configure();
		$prefs->destroy();
	});
	$prefs->show();
	return;
}

sub close_program {
	my $self = shift;
	$self->save_options();
	if ($main::INSTANCES > 1) {
		$self->{app}->hide_all();
		$main::INSTANCES--;
		return;
	} else {
		exit;
	}
}

1;
