# $Id: Preferences.pm,v 1.6 2003/07/19 16:24:33 jodrell Exp $
package Greenwich::Preferences;

use strict;

sub new {
	my $self		= {};
	$self->{PACKAGE}	= shift;
	bless($self, $self->{PACKAGE});
	my %args		= @_;
	$self->{lexicon}	= $args{lexicon};
	$self->{options}	= $args{options};

	$self->{window} = Gtk2::Dialog->new();
	$self->{window}->set_position('center');
	$self->{window}->set_title($self->string('prefstitle'));
	$self->{window}->signal_connect('destroy', sub { $self->destroy() });
	$self->{window}->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($Greenwich::ICONFILE));
	$self->{window}->set_border_width(8);

	$self->{netbox} = Gtk2::HBox->new(0, 0);
	$self->{netbox}->set_border_width(8);
	$self->{netbox}->set_spacing(8);
	$self->{controls}{timeout} = Gtk2::SpinButton->new(Gtk2::Adjustment->new(10, 0, 1000, 1, 10, 0), 1, 0);
	$self->{controls}{timeout}->set_numeric(1);
	$self->{netbox}->pack_start($self->label('timeout'),	1, 1, 0);
	$self->{netbox}->pack_start($self->{controls}{timeout},	0, 0, 0);

	$self->{controls}{recursive} = Gtk2::CheckButton->new($self->string('dorecursive'));
	$self->{controls}{recursive}->set_border_width(8);

	$self->{window}->vbox()->set_border_width(8);
	$self->{window}->vbox()->set_spacing(8);
	$self->{window}->action_area()->set_border_width(0);
	$self->{window}->action_area()->set_spacing(8);

	$self->{labels}{net}	= Gtk2::Label->new;
	$self->{labels}{net}->set_alignment(0, 0);
	$self->{labels}{net}->set_markup('<b>'.$self->string('netlabel').'</b>');
	$self->{labels}{recurs}	= Gtk2::Label->new;
	$self->{labels}{recurs}->set_alignment(0, 0);
	$self->{labels}{recurs}->set_markup('<b>'.$self->string('recursivelabel').'</b>');

	$self->{window}->vbox->pack_start($self->{labels}{net}, 0, 0, 0);
	$self->{window}->vbox->pack_start($self->{netbox}, 0, 0, 0);
	$self->{window}->vbox->pack_start($self->{labels}{recurs}, 0, 0, 0);
	$self->{window}->vbox->pack_start($self->{controls}{recursive}, 0, 0, 0);

	$self->{buttons}{ok_button}	= Gtk2::Button->new_from_stock('gtk-ok');
	$self->{buttons}{cancel_button}	= Gtk2::Button->new_from_stock('gtk-cancel');
	$self->{buttons}{cancel_button}->signal_connect('clicked', sub { $self->destroy() });

	$self->{window}->action_area->pack_end($self->{buttons}{cancel_button}, 0, 0, 0);
	$self->{window}->action_area->pack_end($self->{buttons}{ok_button}, 0, 0, 0);

	$self->configure();

	return $self;
}

sub string {
	my ($self, $str) = @_;
	return $self->{lexicon}->maketext($str);
}

sub label {
	my ($self, $str) = @_;
	return Gtk2::Label->new($self->string($str));
}

sub ok_button {
	my $self = shift;
	return $self->{buttons}{ok_button};
}

sub show {
	my $self = shift;
	$self->{window}->show_all();
}

sub destroy {
	my $self = shift;
	$self->{window}->destroy();
	return;
}

sub configure {
	my $self = shift;
	$self->{controls}{timeout}->set_value($self->{options}{timeout});
	$self->{controls}{recursive}->set_active(1) if ($self->{options}{follow} eq 'true');
	return;
}

sub get_options {
	my $self = shift;
	$self->{options}{timeout}	= $self->{controls}{timeout}->get_value_as_int();
	$self->{options}{follow}	= ($self->{controls}{recursive}->get_active() ? 'true' : 'false');
	return %{$self->{options}};
}

1;
