# $Id: ServerList.pm,v 1.5 2003/11/28 14:42:27 jodrell Exp $
package Greenwich::ServerList;

use strict;

sub new {
	my $self		= {};
	my ($PACKAGE, %args)	= @_;
	$self->{lexicon}	= $args{lexicon};
	$self->{app}		= $args{app};
	$self->{PACKAGE}	= $PACKAGE;
	bless($self, $self->{PACKAGE});
	$self->{window} = Gtk2::Dialog->new();
	$self->{window}->set_default_size(300, 300);
	$self->{window}->set_title($self->string('serverlist'));
	$self->{window}->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($Greenwich::ICONFILE));
	$self->{window}->set_border_width(8);
	$self->{window}->signal_connect('destroy', sub { $self->destroy() });

	$self->{buttons}{ok_button} = Gtk2::Button->new_from_stock('gtk-ok');

	$self->{buttons}{cancel_button} = Gtk2::Button->new_from_stock('gtk-cancel');
	$self->{buttons}{cancel_button}->signal_connect('clicked', sub { $self->destroy() });


	$self->{store} = Gtk2::ListStore->new('Glib::String');
	$self->{renderer} = Gtk2::CellRendererText->new;
	$self->{column} = Gtk2::TreeViewColumn->new_with_attributes('Server', $self->{renderer}, text => 0);
	$self->{list} = Gtk2::TreeView->new($self->{store});
	$self->{list}->append_column($self->{column});

	$self->update_list;

	$self->{scrolled_window} = Gtk2::ScrolledWindow->new(undef, undef);
	$self->{scrolled_window}->set_shadow_type('in');
	$self->{scrolled_window}->set_policy('automatic', 'automatic');
	$self->{scrolled_window}->add($self->{list});

	$self->{button_box} = Gtk2::HBox->new(0, 0);

	$self->{buttons}{add_button} = Gtk2::Button->new_from_stock('gtk-add');
	$self->{buttons}{add_button}->signal_connect(
		'clicked',
		sub {
			my $addWin = Gtk2::Dialog->new();
			$addWin->set_position('center');
			$addWin->set_title($self->string('addserver'));
			$addWin->set_icon(Gtk2::Gdk::Pixbuf->new_from_file($Greenwich::ICONFILE));
			my $okButton = Gtk2::Button->new_from_stock('gtk-ok');
			my $serverName = Gtk2::Entry->new();
			$serverName->signal_connect(
				'activate',
				sub {
					my $server = $serverName->get_text();
					push(@{$self->{servers}}, $server);
					$self->update_list();
					$addWin->destroy();
				}
			);
			$addWin->vbox()->pack_start(Gtk2::Label->new($self->string('entername')), 0, 0, 0);
			$addWin->vbox()->pack_start($serverName, 0, 0, 0);
			$okButton->signal_connect(
				'clicked',
				sub {
					my $server = $serverName->get_text();
					push(@{$self->{servers}}, $server);
					$self->update_list();
					$addWin->destroy();
				}
			);
			my $cancelButton = Gtk2::Button->new_from_stock('gtk-cancel');
			$cancelButton->signal_connect('clicked', sub { $addWin->destroy() });
			$addWin->vbox()->set_border_width(8);
			$addWin->vbox()->set_spacing(8);
			$addWin->action_area()->set_border_width(0);
			$addWin->action_area()->pack_start($cancelButton, 1, 1, 0);
			$addWin->action_area()->pack_start($okButton, 1, 1, 0);
			$addWin->show_all();
			$serverName->grab_focus();
		}
	);
	$self->{buttons}{del_button} = Gtk2::Button->new_from_stock('gtk-remove');
	$self->{buttons}{del_button}->signal_connect(
		'clicked',
		sub {
			my (undef, $iter) = $self->{list}->get_selection->get_selected;
			return undef unless (defined($iter));
			my $idx = ($self->{store}->get_path($iter)->get_indices)[0];
			$self->{store}->remove($iter);
			splice(@{$self->{servers}}, $idx, 1);
		}
	);

	$self->{window}->action_area()->set_border_width(0);
	$self->{window}->vbox()->set_border_width(8);
	$self->{window}->vbox()->set_spacing(8);
	$self->{button_box}->set_spacing(8);

	$self->{button_box}->pack_start($self->{buttons}{add_button},	1, 1, 0);
	$self->{button_box}->pack_start($self->{buttons}{del_button},	1, 1, 0);

	$self->{window}->vbox()->pack_start($self->{scrolled_window}, 1, 1, 0);
	$self->{window}->vbox()->pack_start($self->{button_box}, 0, 0, 0);

	$self->{window}->action_area()->pack_start($self->{buttons}{cancel_button},	1, 1, 0);
	$self->{window}->action_area()->pack_start($self->{buttons}{ok_button},		1, 1, 0);

	return $self;
}

sub set_servers {
	my $self = shift;
	my @servers = @_;
	$self->{servers} = \@servers;
	$self->update_list();
	return;
}

sub update_list {
	my $self = shift;
	$self->{store}->clear;
	foreach my $server (@{$self->{servers}}) {
		my $iter = $self->{store}->append;
		$self->{store}->set($iter, 0, $server);
	}
	return 1;
}

sub servers {
	my $self = shift;
	return grep { defined } @{$self->{servers}};
}

sub string {
	my ($self, $str) = @_;
	return $self->{lexicon}->maketext($str);
}

sub show {
	my $self = shift;
	$self->{window}->show_all();
	$self->{buttons}{ok_button}->grab_focus();
	return;
}

sub ok_button {
	my $self = shift;
	return $self->{buttons}{ok_button};
}

sub destroy {
	my $self = shift;
	$self->{window}->destroy();
}

1;
